@@ -140,19 +140,16 @@ struct RsvgHandlePrivate {
 
     RsvgLoad *load;
 
-    gboolean is_disposed;
-
     RsvgSizeFunc size_func;
     gpointer user_data;
     GDestroyNotify user_data_destroy;
 
-    GPtrArray *all_nodes;
-
-    RsvgDefs *defs; /* lookup table for nodes that have an id="foo" attribute */
     /* this is the root level of the displayable tree, essentially what the
        file is converted into at the end */
     RsvgNode *treebase;
 
+    RsvgDefs *defs; /* lookup table for nodes that have an id="foo" attribute */
+
     GHashTable *css_props;
 
     GCancellable *cancellable;
@@ -175,31 +172,6 @@ struct RsvgHandlePrivate {
 #endif
 };
 
-/* Keep this in sync with rust/src/length.rs:LengthUnit */
-typedef enum {
-    LENGTH_UNIT_DEFAULT,
-    LENGTH_UNIT_PERCENT,
-    LENGTH_UNIT_FONT_EM,
-    LENGTH_UNIT_FONT_EX,
-    LENGTH_UNIT_INCH,
-    LENGTH_UNIT_RELATIVE_LARGER,
-    LENGTH_UNIT_RELATIVE_SMALLER
-} LengthUnit;
-
-/* Keep this in sync with rust/src/length.rs:LengthDir */
-typedef enum {
-    LENGTH_DIR_HORIZONTAL,
-    LENGTH_DIR_VERTICAL,
-    LENGTH_DIR_BOTH
-} LengthDir;
-
-/* Keep this in sync with rust/src/length.rs:RsvgLength */
-typedef struct {
-    double length;
-    LengthUnit unit;
-    LengthDir dir;
-} RsvgLength;
-
 typedef enum {
     userSpaceOnUse,
     objectBoundingBox
@@ -261,9 +233,6 @@ typedef enum {
     RSVG_NODE_TYPE_FILTER_PRIMITIVE_LAST                /* just a marker; not a valid type */
 } RsvgNodeType;
 
-/* Defined in rsvg_internals/src/state.rs */
-typedef struct _RsvgComputedValues *RsvgComputedValues;
-
 typedef void (* CNodeSetAtts) (RsvgNode *node, gpointer impl, RsvgHandle *handle, RsvgPropertyBag pbag);
 typedef void (* CNodeFree) (gpointer impl);
 
@@ -325,17 +294,9 @@ gboolean rsvg_node_children_iter_next_back (RsvgNodeChildrenIter *iter,
 G_GNUC_INTERNAL
 void rsvg_node_children_iter_end (RsvgNodeChildrenIter *iter);
 
-/* Implemented in rust/src/state.rs */
+/* Implemented in rsvg_internals/src/structure.rs */
 G_GNUC_INTERNAL
-guint32 rsvg_computed_values_get_flood_color_argb (RsvgComputedValues *values);
-
-/* Implemented in rust/src/state.rs */
-G_GNUC_INTERNAL
-guint8 rsvg_computed_values_get_flood_opacity (RsvgComputedValues *values);
-
-/* Implemented in rust/src/state.rs */
-G_GNUC_INTERNAL
-guint32 rsvg_computed_values_get_lighting_color_argb (RsvgComputedValues *values);
+gboolean rsvg_node_svg_get_size (RsvgNode *node, double dpi_x, double dpi_y, int *out_width, int *out_height);
 
 typedef void (*RsvgPropertyBagEnumFunc) (const char *key, const char *value, gpointer user_data);
 
@@ -364,12 +325,6 @@ gboolean rsvg_property_bag_iter_next (RsvgPropertyBagIter *iter,
 G_GNUC_INTERNAL
 void rsvg_property_bag_iter_end (RsvgPropertyBagIter *iter);
 
-/* for some reason this one's public... */
-GdkPixbuf *rsvg_pixbuf_from_data_with_size_data (const guchar * buff,
-                                                 size_t len,
-                                                 gpointer data,
-                                                 const char *base_uri, GError ** error);
-
 /* Implemented in rust/src/cond.rs */
 G_GNUC_INTERNAL
 gboolean rsvg_cond_check_required_features (const char *value);
@@ -390,21 +345,6 @@ GdkPixbuf *rsvg_cairo_surface_to_pixbuf (cairo_surface_t *surface);
 G_GNUC_INTERNAL
 cairo_surface_t *rsvg_cairo_surface_new_from_href (RsvgHandle *handle, const char *href, GError ** error);
 
-/* This is implemented in rust/src/length.rs */
-G_GNUC_INTERNAL
-double rsvg_length_normalize (const RsvgLength *length, RsvgComputedValues *values, RsvgDrawingCtx * ctx);
-
-/* This is implemented in rust/src/length.rs */
-G_GNUC_INTERNAL
-double rsvg_length_hand_normalize (const RsvgLength *length,
-                                   double pixels_per_inch,
-                                   double width_or_height,
-                                   double font_size);
-
-/* Implemented in rust/src/length.rs */
-G_GNUC_INTERNAL
-RsvgLength rsvg_length_parse (const char *str, LengthDir dir);
-
 /* Defined in rsvg_internals/src/drawing_ctx.rs */
 G_GNUC_INTERNAL
 RsvgDrawingCtx *rsvg_drawing_ctx_new (cairo_t *cr,
@@ -448,18 +388,28 @@ void rsvg_return_if_fail_warning (const char *pretty_function,
 G_GNUC_INTERNAL
 RsvgNode *rsvg_load_destroy (RsvgLoad *load) G_GNUC_WARN_UNUSED_RESULT;
 
+/* Defined in rsvg_internals/src/defs.rs */
 G_GNUC_INTERNAL
-void rsvg_add_node_to_handle (RsvgHandle *handle, RsvgNode *node);
+RsvgDefs *rsvg_defs_new (RsvgHandle *handle);
+
+/* Defined in rsvg_internals/src/defs.rs */
+G_GNUC_INTERNAL
+void rsvg_defs_free (RsvgDefs *defs);
+
+/* Defined in rsvg_internals/src/defs.rs */
+/* for some reason this one's public... */
+RsvgNode *rsvg_defs_lookup (const RsvgDefs * defs, const char *name);
+
+G_GNUC_INTERNAL
+RsvgDefs *rsvg_handle_get_defs (RsvgHandle *handle);
 
 G_GNUC_INTERNAL
 char *rsvg_handle_resolve_uri (RsvgHandle *handle,
                                const char *uri);
 
 G_GNUC_INTERNAL
-void rsvg_handle_cascade (RsvgHandle *handle);
-
-G_GNUC_INTERNAL
-void rsvg_handle_update_font_map_for_testing (RsvgHandle *handle);
+RsvgHandle *rsvg_handle_load_extern (RsvgHandle *handle,
+                                     const char *uri);
 
 G_GNUC_INTERNAL
 gboolean rsvg_allow_load (GFile       *base_gfile,
