@@ -316,10 +316,8 @@ main (int argc, char **argv)
         if (i == 0) {
             struct RsvgSizeCallbackData size_data;
 
-            if (!rsvg_handle_get_dimensions_sub (rsvg, &dimensions, export_lookup_id)) {
+            if (!rsvg_handle_get_dimensions_sub (rsvg, &dimensions, export_lookup_id))
                 g_printerr ("Could not get dimensions for file %s\n", args[i]);
-                exit (1);
-            }
 
             unscaled_width = dimensions.width;
             unscaled_height = dimensions.height;
@@ -435,7 +433,7 @@ main (int argc, char **argv)
         if (background_color_str && g_ascii_strcasecmp(background_color_str, "none") != 0) {
             RsvgCssColorSpec spec;
 
-            spec = rsvg_css_parse_color_ (background_color_str, ALLOW_INHERIT_NO, ALLOW_CURRENT_COLOR_NO);
+            spec = rsvg_css_parse_color_ (background_color_str);
             if (spec.kind == RSVG_CSS_COLOR_SPEC_ARGB) {
                 background_color = spec.argb;
             } else {
@@ -467,10 +465,7 @@ main (int argc, char **argv)
         cairo_scale (cr,
                      scaled_width / unscaled_width,
                      scaled_height / unscaled_height);
-        if (!rsvg_handle_render_cairo_sub (rsvg, cr, export_lookup_id)) {
-            g_printerr ("Could not render file %s\n", args[i]);
-            exit (1);
-        }
+        rsvg_handle_render_cairo_sub (rsvg, cr, export_lookup_id);
 
         g_free (export_lookup_id);
 
