@@ -32,8 +32,7 @@ impl Parse for CoordUnits {
                         cow.as_ref().to_string(),
                     ))),
                 ),
-            })
-            .map_err(|_| {
+            }).map_err(|_| {
                 AttributeError::Parse(ParseError::new(
                     "expected 'userSpaceOnUse' or 'objectBoundingBox'",
                 ))
