@@ -14,7 +14,7 @@ use std::rc::Rc;
 use bbox::BoundingBox;
 use clip_path::{ClipPathUnits, NodeClipPath};
 use coord_units::CoordUnits;
-use defs::{self, RsvgDefs};
+use defs::{Defs, RsvgDefs};
 use filters;
 use float_eq_cairo::ApproxEqCairo;
 use length::Dasharray;
@@ -39,7 +39,7 @@ use viewbox::ViewBox;
 
 pub enum RsvgDrawingCtx {}
 
-pub struct DrawingCtx {
+pub struct DrawingCtx<'a> {
     rect: cairo::Rectangle,
     dpi_x: f64,
     dpi_y: f64,
@@ -58,13 +58,13 @@ pub struct DrawingCtx {
 
     drawsub_stack: Vec<RsvgNode>,
 
-    defs: *const RsvgDefs,
+    defs: RefCell<&'a mut Defs>,
     acquired_nodes: RefCell<Vec<RsvgNode>>,
 
     is_testing: bool,
 }
 
-impl<'a> DrawingCtx {
+impl<'a> DrawingCtx<'a> {
     pub fn new(
         cr: cairo::Context,
         width: f64,
@@ -73,7 +73,7 @@ impl<'a> DrawingCtx {
         vb_height: f64,
         dpi_x: f64,
         dpi_y: f64,
-        defs: *const RsvgDefs,
+        defs: &mut Defs,
         is_testing: bool,
     ) -> DrawingCtx {
         let mut affine = cr.get_matrix();
@@ -83,7 +83,7 @@ impl<'a> DrawingCtx {
             width,
             height,
         }.transform(&affine)
-            .outer();
+        .outer();
 
         // scale according to size set by size_func callback
         let mut scale = cairo::Matrix::identity();
@@ -109,7 +109,7 @@ impl<'a> DrawingCtx {
             bbox: BoundingBox::new(&affine),
             bbox_stack: Vec::new(),
             drawsub_stack: Vec::new(),
-            defs,
+            defs: RefCell::new(defs),
             acquired_nodes: RefCell::new(Vec::new()),
             is_testing,
         }
@@ -195,7 +195,7 @@ impl<'a> DrawingCtx {
     // acquire it again.  If you acquire a node "#foo" and don't release it before
     // trying to acquire "foo" again, you will obtain a %NULL the second time.
     pub fn get_acquired_node(&mut self, url: &str) -> Option<AcquiredNode> {
-        if let Some(node) = defs::lookup(self.defs, url) {
+        if let Some(node) = self.defs.borrow_mut().lookup(url) {
             if !self.acquired_nodes_contains(node) {
                 self.acquired_nodes.borrow_mut().push(node.clone());
                 return Some(AcquiredNode(&self.acquired_nodes as *const _, node.clone()));
@@ -301,39 +301,35 @@ impl<'a> DrawingCtx {
                 && comp_op == CompOp::SrcOver
                 && enable_background == EnableBackground::Accumulate);
 
-            let child_surface = {
-                if needs_temporary_surface {
-                    // FIXME: in the following, we unwrap() the result of
-                    // ImageSurface::create().  We have to decide how to handle
-                    // out-of-memory here.
-                    let surface = cairo::ImageSurface::create(
-                        cairo::Format::ARgb32,
-                        self.rect.width as i32,
-                        self.rect.height as i32,
-                    ).unwrap();
+            if needs_temporary_surface {
+                // FIXME: in the following, we unwrap() the result of
+                // ImageSurface::create().  We have to decide how to handle
+                // out-of-memory here.
+                let surface = cairo::ImageSurface::create(
+                    cairo::Format::ARgb32,
+                    self.rect.width as i32,
+                    self.rect.height as i32,
+                ).unwrap();
 
-                    if filter.is_some() {
-                        self.surfaces_stack.push(surface.clone());
-                    }
-
-                    let cr = cairo::Context::new(&surface);
-                    cr.set_matrix(affine);
-
-                    self.cr_stack.push(self.cr.clone());
-                    self.cr = cr.clone();
-
-                    self.bbox_stack.push(self.bbox);
-                    self.bbox = BoundingBox::new(&affine);
-
-                    surface
-                } else {
-                    cairo::ImageSurface::from(original_cr.get_target()).unwrap()
+                if filter.is_some() {
+                    self.surfaces_stack.push(surface.clone());
                 }
-            };
+
+                let cr = cairo::Context::new(&surface);
+                cr.set_matrix(affine);
+
+                self.cr_stack.push(self.cr.clone());
+                self.cr = cr.clone();
+
+                self.bbox_stack.push(self.bbox);
+                self.bbox = BoundingBox::new(&affine);
+            }
 
             draw_fn(self);
 
             if needs_temporary_surface {
+                let child_surface = cairo::ImageSurface::from(self.cr.get_target()).unwrap();
+
                 let filter_result_surface = filter
                     .and_then(|_| {
                         // About the following unwrap(), see the FIXME above.  We should be pushing
@@ -365,8 +361,7 @@ impl<'a> DrawingCtx {
                                 )
                             }
                         }
-                    })
-                    .or(Some(child_surface))
+                    }).or(Some(child_surface))
                     .unwrap();
 
                 self.cr = self.cr_stack.pop().unwrap();
@@ -973,9 +968,12 @@ pub extern "C" fn rsvg_drawing_ctx_new(
     vb_height: libc::c_double,
     dpi_x: libc::c_double,
     dpi_y: libc::c_double,
-    defs: *const RsvgDefs,
+    defs: *mut RsvgDefs,
     is_testing: glib_sys::gboolean,
 ) -> *mut RsvgDrawingCtx {
+    assert!(!defs.is_null());
+    let defs = unsafe { &mut *(defs as *mut Defs) };
+
     Box::into_raw(Box::new(DrawingCtx::new(
         unsafe { from_glib_none(cr) },
         f64::from(width),
