@@ -83,10 +83,24 @@ impl Filter for Blend {
             .add_input(&input_2)
             .into_irect(draw_ctx);
 
-        let output_surface = input_2
-            .surface()
-            .copy_surface(bounds)
-            .map_err(FilterError::IntermediateSurfaceCreation)?;
+        // If we're combining two alpha-only surfaces, the result is alpha-only. Otherwise the
+        // result is whatever the non-alpha-only type we're working on (which can be either sRGB or
+        // linear sRGB depending on color-interpolation-filters).
+        let surface_type = if input.surface().is_alpha_only() {
+            input_2.surface().surface_type()
+        } else {
+            if !input_2.surface().is_alpha_only() {
+                // All surface types should match (this is enforced by get_input()).
+                assert_eq!(
+                    input_2.surface().surface_type(),
+                    input.surface().surface_type()
+                );
+            }
+
+            input.surface().surface_type()
+        };
+
+        let output_surface = input_2.surface().copy_surface(bounds)?;
         {
             let cr = cairo::Context::new(&output_surface);
             cr.rectangle(
@@ -105,8 +119,7 @@ impl Filter for Blend {
         Ok(FilterResult {
             name: self.base.result.borrow().clone(),
             output: FilterOutput {
-                surface: SharedImageSurface::new(output_surface)
-                    .map_err(FilterError::BadIntermediateSurfaceStatus)?,
+                surface: SharedImageSurface::new(output_surface, surface_type)?,
                 bounds,
             },
         })
