@@ -290,7 +290,7 @@ impl Filter for ComponentTransfer {
             cairo::Format::ARgb32,
             ctx.source_graphic().width(),
             ctx.source_graphic().height(),
-        ).map_err(FilterError::IntermediateSurfaceCreation)?;
+        )?;
 
         // Enumerate all child <feFuncX> nodes.
         let functions = node
@@ -372,8 +372,7 @@ impl Filter for ComponentTransfer {
         Ok(FilterResult {
             name: self.base.result.borrow().clone(),
             output: FilterOutput {
-                surface: SharedImageSurface::new(output_surface)
-                    .map_err(FilterError::BadIntermediateSurfaceStatus)?,
+                surface: SharedImageSurface::new(output_surface, input.surface().surface_type())?,
                 bounds,
             },
         })
