@@ -111,12 +111,29 @@ impl Filter for Composite {
             .add_input(&input_2)
             .into_irect(draw_ctx);
 
+        // If we're combining two alpha-only surfaces, the result is alpha-only. Otherwise the
+        // result is whatever the non-alpha-only type we're working on (which can be either sRGB or
+        // linear sRGB depending on color-interpolation-filters).
+        let surface_type = if input.surface().is_alpha_only() {
+            input_2.surface().surface_type()
+        } else {
+            if !input_2.surface().is_alpha_only() {
+                // All surface types should match (this is enforced by get_input()).
+                assert_eq!(
+                    input_2.surface().surface_type(),
+                    input.surface().surface_type()
+                );
+            }
+
+            input.surface().surface_type()
+        };
+
         let output_surface = if self.operator.get() == Operator::Arithmetic {
             let mut output_surface = ImageSurface::create(
                 cairo::Format::ARgb32,
                 input.surface().width(),
                 input.surface().height(),
-            ).map_err(FilterError::IntermediateSurfaceCreation)?;
+            )?;
 
             let output_stride = output_surface.get_stride() as usize;
             {
@@ -162,10 +179,7 @@ impl Filter for Composite {
 
             output_surface
         } else {
-            let output_surface = input_2
-                .surface()
-                .copy_surface(bounds)
-                .map_err(FilterError::IntermediateSurfaceCreation)?;
+            let output_surface = input_2.surface().copy_surface(bounds)?;
 
             let cr = cairo::Context::new(&output_surface);
             cr.rectangle(
@@ -186,8 +200,7 @@ impl Filter for Composite {
         Ok(FilterResult {
             name: self.base.result.borrow().clone(),
             output: FilterOutput {
-                surface: SharedImageSurface::new(output_surface)
-                    .map_err(FilterError::BadIntermediateSurfaceStatus)?,
+                surface: SharedImageSurface::new(output_surface, surface_type)?,
                 bounds,
             },
         })
@@ -220,8 +233,7 @@ impl Parse for Operator {
                         cow.as_ref().to_string(),
                     ))),
                 ),
-            })
-            .map_err(|_| AttributeError::Value("invalid operator value".to_string()))
+            }).map_err(|_| AttributeError::Value("invalid operator value".to_string()))
     }
 }
 
