@@ -7,11 +7,10 @@ use cairo::{self, MatrixTrait};
 use bbox::BoundingBox;
 use coord_units::CoordUnits;
 use drawing_ctx::DrawingCtx;
-use length::RsvgLength;
+use length::Length;
 use node::RsvgNode;
 use paint_server::{self, PaintServer};
-use srgb::{linearize_surface, unlinearize_surface};
-use surface_utils::shared_surface::SharedImageSurface;
+use surface_utils::shared_surface::{SharedImageSurface, SurfaceType};
 use unitinterval::UnitInterval;
 
 use super::error::FilterError;
@@ -294,10 +293,10 @@ impl FilterContext {
 
     /// Returns the surface containing the source graphic alpha.
     #[inline]
-    pub fn source_alpha(&self, bounds: IRect) -> Result<cairo::ImageSurface, FilterError> {
+    pub fn source_alpha(&self, bounds: IRect) -> Result<SharedImageSurface, FilterError> {
         self.source_surface
             .extract_alpha(bounds)
-            .map_err(FilterError::IntermediateSurfaceCreation)
+            .map_err(FilterError::CairoError)
     }
 
     /// Computes and returns the background image snapshot.
@@ -354,10 +353,10 @@ impl FilterContext {
 
         *bg = Some(
             self.compute_background_image(draw_ctx)
-                .map_err(FilterError::IntermediateSurfaceCreation)
+                .map_err(FilterError::CairoError)
                 .and_then(|surface| {
-                    SharedImageSurface::new(surface)
-                        .map_err(FilterError::BadIntermediateSurfaceStatus)
+                    SharedImageSurface::new(surface, SurfaceType::SRgb)
+                        .map_err(FilterError::CairoError)
                 }),
         );
 
@@ -371,10 +370,10 @@ impl FilterContext {
         &self,
         draw_ctx: &DrawingCtx,
         bounds: IRect,
-    ) -> Result<cairo::ImageSurface, FilterError> {
+    ) -> Result<SharedImageSurface, FilterError> {
         self.background_image(draw_ctx)?
             .extract_alpha(bounds)
-            .map_err(FilterError::IntermediateSurfaceCreation)
+            .map_err(FilterError::CairoError)
     }
 
     /// Returns the output of the filter primitive by its result name.
@@ -385,37 +384,31 @@ impl FilterContext {
 
     /// Converts this `FilterContext` into the surface corresponding to the output of the filter
     /// chain.
+    ///
+    /// The returned surface is in the sRGB color space.
+    // TODO: sRGB conversion should probably be done by the caller.
     #[inline]
     pub fn into_output(self) -> Result<SharedImageSurface, FilterError> {
-        match self.last_result.map(|FilterOutput { surface, .. }| surface) {
-            Some(surface) => Ok(surface),
+        match self.last_result {
+            Some(FilterOutput { surface, bounds }) => {
+                surface.to_srgb(bounds).map_err(FilterError::CairoError)
+            }
             None => {
                 let empty_surface = cairo::ImageSurface::create(
                     cairo::Format::ARgb32,
                     self.source_surface.width(),
                     self.source_surface.height(),
-                ).map_err(FilterError::IntermediateSurfaceCreation)?;
+                ).map_err(FilterError::CairoError)?;
 
-                SharedImageSurface::new(empty_surface)
-                    .map_err(FilterError::BadIntermediateSurfaceStatus)
+                SharedImageSurface::new(empty_surface, SurfaceType::AlphaOnly)
+                    .map_err(FilterError::CairoError)
             }
         }
     }
 
     /// Stores a filter primitive result into the context.
     #[inline]
-    pub fn store_result(&mut self, mut result: FilterResult) -> Result<(), FilterError> {
-        // Unlinearize the surface if needed.
-        if self.processing_linear_rgb {
-            result.output.surface =
-                unlinearize_surface(&result.output.surface, result.output.bounds)
-                    .map_err(FilterError::IntermediateSurfaceCreation)
-                    .and_then(|surface| {
-                        SharedImageSurface::new(surface)
-                            .map_err(FilterError::BadIntermediateSurfaceStatus)
-                    })?;
-        }
-
+    pub fn store_result(&mut self, result: FilterResult) -> Result<(), FilterError> {
         if let Some(name) = result.name {
             self.previous_results.insert(name, result.output.clone());
         }
@@ -440,7 +433,7 @@ impl FilterContext {
     pub fn with_primitive_units<F, T>(&self, draw_ctx: &mut DrawingCtx, f: F) -> T
     // TODO: Get rid of this Box? Can't just impl Trait because Rust cannot do higher-ranked types.
     where
-        for<'b> F: FnOnce(Box<Fn(&RsvgLength) -> f64 + 'b>) -> T,
+        for<'b> F: FnOnce(Box<Fn(&Length) -> f64 + 'b>) -> T,
     {
         // Filters use the properties of the target node.
         let cascaded = self.node_being_filtered.get_cascaded_values();
@@ -451,12 +444,12 @@ impl FilterContext {
         // See comments in compute_effects_region() for how this works.
         if filter.primitiveunits.get() == CoordUnits::ObjectBoundingBox {
             draw_ctx.push_view_box(1.0, 1.0);
-            let rv = f(Box::new(RsvgLength::get_unitless));
+            let rv = f(Box::new(Length::get_unitless));
             draw_ctx.pop_view_box();
 
             rv
         } else {
-            f(Box::new(|length: &RsvgLength| {
+            f(Box::new(|length: &Length| {
                 length.normalize(values, draw_ctx)
             }))
         }
@@ -524,10 +517,6 @@ impl FilterContext {
             Input::SourceGraphic => Ok(FilterInput::StandardInput(self.source_graphic().clone())),
             Input::SourceAlpha => self
                 .source_alpha(self.effects_region().rect.unwrap().into())
-                .and_then(|surface| {
-                    SharedImageSurface::new(surface)
-                        .map_err(FilterError::BadIntermediateSurfaceStatus)
-                })
                 .map(FilterInput::StandardInput),
             Input::BackgroundImage => self
                 .background_image(draw_ctx)
@@ -535,28 +524,22 @@ impl FilterContext {
                 .map(FilterInput::StandardInput),
             Input::BackgroundAlpha => self
                 .background_alpha(draw_ctx, self.effects_region().rect.unwrap().into())
-                .and_then(|surface| {
-                    SharedImageSurface::new(surface)
-                        .map_err(FilterError::BadIntermediateSurfaceStatus)
-                })
                 .map(FilterInput::StandardInput),
 
             Input::FillPaint => self
                 .get_paint_server_surface(draw_ctx, &values.fill.0, values.fill_opacity.0)
-                .map_err(FilterError::IntermediateSurfaceCreation)
+                .map_err(FilterError::CairoError)
                 .and_then(|surface| {
-                    SharedImageSurface::new(surface)
-                        .map_err(FilterError::BadIntermediateSurfaceStatus)
-                })
-                .map(FilterInput::StandardInput),
+                    SharedImageSurface::new(surface, SurfaceType::SRgb)
+                        .map_err(FilterError::CairoError)
+                }).map(FilterInput::StandardInput),
             Input::StrokePaint => self
                 .get_paint_server_surface(draw_ctx, &values.stroke.0, values.stroke_opacity.0)
-                .map_err(FilterError::IntermediateSurfaceCreation)
+                .map_err(FilterError::CairoError)
                 .and_then(|surface| {
-                    SharedImageSurface::new(surface)
-                        .map_err(FilterError::BadIntermediateSurfaceStatus)
-                })
-                .map(FilterInput::StandardInput),
+                    SharedImageSurface::new(surface, SurfaceType::SRgb)
+                        .map_err(FilterError::CairoError)
+                }).map(FilterInput::StandardInput),
 
             Input::FilterOutput(ref name) => self
                 .filter_output(name)
@@ -586,12 +569,9 @@ impl FilterContext {
                 }) => (surface, *bounds),
             };
 
-            linearize_surface(surface, bounds)
-                .map_err(FilterError::IntermediateSurfaceCreation)
-                .and_then(|surface| {
-                    SharedImageSurface::new(surface)
-                        .map_err(FilterError::BadIntermediateSurfaceStatus)
-                })
+            surface
+                .to_linear_rgb(bounds)
+                .map_err(FilterError::CairoError)
                 .map(|surface| match raw {
                     FilterInput::StandardInput(_) => FilterInput::StandardInput(surface),
                     FilterInput::PrimitiveOutput(ref output) => {
@@ -685,8 +665,8 @@ mod tests {
             }
         }
 
-        let surface = SharedImageSurface::new(surface).unwrap();
-        let alpha = SharedImageSurface::new(surface.extract_alpha(BOUNDS).unwrap()).unwrap();
+        let surface = SharedImageSurface::new(surface, SurfaceType::SRgb).unwrap();
+        let alpha = surface.extract_alpha(BOUNDS).unwrap();
 
         for (x, y, p, pa) in
             Pixels::new(&surface, FULL_BOUNDS).map(|(x, y, p)| (x, y, p, alpha.get_pixel(x, y)))
