@@ -9,12 +9,12 @@ use coord_units::CoordUnits;
 use drawing_ctx::DrawingCtx;
 use error::AttributeError;
 use handle::RsvgHandle;
-use length::{LengthDir, LengthUnit, RsvgLength};
+use length::{Length, LengthDir, LengthUnit};
 use node::{NodeResult, NodeTrait, NodeType, RsvgNode};
 use parsers::{parse_and_validate, ParseError};
 use property_bag::PropertyBag;
 use state::ColorInterpolationFilters;
-use surface_utils::shared_surface::SharedImageSurface;
+use surface_utils::shared_surface::{SharedImageSurface, SurfaceType};
 
 mod bounds;
 use self::bounds::BoundsBuilder;
@@ -70,10 +70,10 @@ trait Filter: NodeTrait {
 
 /// The base filter primitive node containing common properties.
 struct Primitive {
-    x: Cell<Option<RsvgLength>>,
-    y: Cell<Option<RsvgLength>>,
-    width: Cell<Option<RsvgLength>>,
-    height: Cell<Option<RsvgLength>>,
+    x: Cell<Option<Length>>,
+    y: Cell<Option<Length>>,
+    width: Cell<Option<Length>>,
+    height: Cell<Option<Length>>,
     result: RefCell<Option<String>>,
 }
 
@@ -120,11 +120,10 @@ impl NodeTrait for Primitive {
                 } else {
                     None
                 }
-            })
-            .unwrap_or(CoordUnits::UserSpaceOnUse);
+            }).unwrap_or(CoordUnits::UserSpaceOnUse);
 
         let no_units_allowed = primitiveunits == CoordUnits::ObjectBoundingBox;
-        let check_units = |length: RsvgLength| {
+        let check_units = |length: Length| {
             if !no_units_allowed {
                 return Ok(length);
             }
@@ -137,7 +136,7 @@ impl NodeTrait for Primitive {
             }
         };
         let check_units_and_ensure_nonnegative =
-            |length: RsvgLength| check_units(length).and_then(RsvgLength::check_nonnegative);
+            |length: Length| check_units(length).and_then(Length::check_nonnegative);
 
         for (_key, attr, value) in pbag.iter() {
             match attr {
@@ -247,7 +246,7 @@ pub fn render(
         cr.set_source_surface(source, 0f64, 0f64);
         cr.paint();
     }
-    let source_surface = SharedImageSurface::new(source_surface).unwrap();
+    let source_surface = SharedImageSurface::new(source_surface, SurfaceType::SRgb).unwrap();
 
     let mut filter_ctx =
         FilterContext::new(filter_node, node_being_filtered, source_surface, draw_ctx);
@@ -329,13 +328,9 @@ pub fn render(
             }
         });
 
-    match filter_ctx.into_output() {
-        Ok(surface) => surface.into_image_surface(),
-        Err(err) => {
-            panic!(
-                "Could not create an empty surface to return from a filter: {}",
-                err
-            );
-        }
-    }
+    filter_ctx
+        .into_output()
+        .expect("could not create an empty surface to return from a filter")
+        .into_image_surface()
+        .expect("could not convert filter output into an ImageSurface")
 }
