@@ -9,7 +9,12 @@ use handle::RsvgHandle;
 use node::{NodeResult, NodeTrait, RsvgNode};
 use parsers::{self, ParseError};
 use property_bag::PropertyBag;
-use surface_utils::{shared_surface::SharedImageSurface, ImageSurfaceDataExt, Pixel};
+use state::ColorInterpolationFilters;
+use surface_utils::{
+    shared_surface::{SharedImageSurface, SurfaceType},
+    ImageSurfaceDataExt,
+    Pixel,
+};
 use util::clamp;
 
 use super::context::{FilterContext, FilterOutput, FilterResult};
@@ -89,8 +94,7 @@ impl NodeTrait for Turbulence {
                                 f64::from(i32::min_value()),
                                 f64::from(i32::max_value()),
                             ) as i32
-                        })
-                        .map_err(|err| NodeError::parse_error(attr, err))?,
+                        }).map_err(|err| NodeError::parse_error(attr, err))?,
                 ),
                 Attribute::StitchTiles => self.stitch_tiles.set(StitchTiles::parse(attr, value)?),
                 Attribute::Type => self.type_.set(NoiseType::parse(attr, value)?),
@@ -338,7 +342,7 @@ impl NoiseGenerator {
 impl Filter for Turbulence {
     fn render(
         &self,
-        _node: &RsvgNode,
+        node: &RsvgNode,
         ctx: &FilterContext,
         draw_ctx: &mut DrawingCtx,
     ) -> Result<FilterResult, FilterError> {
@@ -362,7 +366,7 @@ impl Filter for Turbulence {
             cairo::Format::ARgb32,
             ctx.source_graphic().width(),
             ctx.source_graphic().height(),
-        ).map_err(FilterError::IntermediateSurfaceCreation)?;
+        )?;
 
         let output_stride = output_surface.get_stride() as usize;
         {
@@ -401,11 +405,21 @@ impl Filter for Turbulence {
             }
         }
 
+        let cascaded = node.get_cascaded_values();
+        let values = cascaded.get();
+        // The generated color values are in the color space determined by
+        // color-interpolation-filters.
+        let surface_type =
+            if values.color_interpolation_filters == ColorInterpolationFilters::LinearRgb {
+                SurfaceType::LinearRgb
+            } else {
+                SurfaceType::SRgb
+            };
+
         Ok(FilterResult {
             name: self.base.result.borrow().clone(),
             output: FilterOutput {
-                surface: SharedImageSurface::new(output_surface)
-                    .map_err(FilterError::BadIntermediateSurfaceStatus)?,
+                surface: SharedImageSurface::new(output_surface, surface_type)?,
                 bounds,
             },
         })
