@@ -53,8 +53,7 @@ impl Parse for SpreadMethod {
                         cow.as_ref().to_string(),
                     ))),
                 ),
-            })
-            .map_err(|_| {
+            }).map_err(|_| {
                 AttributeError::Parse(ParseError::new("expected 'pad' | 'reflect' | 'repeat'"))
             })
     }
@@ -91,18 +90,18 @@ struct GradientCommon {
 #[derive(Copy, Clone)]
 enum GradientVariant {
     Linear {
-        x1: Option<RsvgLength>,
-        y1: Option<RsvgLength>,
-        x2: Option<RsvgLength>,
-        y2: Option<RsvgLength>,
+        x1: Option<Length>,
+        y1: Option<Length>,
+        x2: Option<Length>,
+        y2: Option<Length>,
     },
 
     Radial {
-        cx: Option<RsvgLength>,
-        cy: Option<RsvgLength>,
-        r: Option<RsvgLength>,
-        fx: Option<RsvgLength>,
-        fy: Option<RsvgLength>,
+        cx: Option<Length>,
+        cy: Option<Length>,
+        r: Option<Length>,
+        fx: Option<Length>,
+        fy: Option<Length>,
     },
 }
 
@@ -249,10 +248,10 @@ impl GradientVariant {
         // https://www.w3.org/TR/SVG/pservers.html#LinearGradients
 
         GradientVariant::Linear {
-            x1: Some(RsvgLength::parse_str("0%", LengthDir::Horizontal).unwrap()),
-            y1: Some(RsvgLength::parse_str("0%", LengthDir::Vertical).unwrap()),
-            x2: Some(RsvgLength::parse_str("100%", LengthDir::Horizontal).unwrap()),
-            y2: Some(RsvgLength::parse_str("0%", LengthDir::Vertical).unwrap()),
+            x1: Some(Length::parse_str("0%", LengthDir::Horizontal).unwrap()),
+            y1: Some(Length::parse_str("0%", LengthDir::Vertical).unwrap()),
+            x2: Some(Length::parse_str("100%", LengthDir::Horizontal).unwrap()),
+            y2: Some(Length::parse_str("0%", LengthDir::Vertical).unwrap()),
         }
     }
 
@@ -260,9 +259,9 @@ impl GradientVariant {
         // https://www.w3.org/TR/SVG/pservers.html#RadialGradients
 
         GradientVariant::Radial {
-            cx: Some(RsvgLength::parse_str("50%", LengthDir::Horizontal).unwrap()),
-            cy: Some(RsvgLength::parse_str("50%", LengthDir::Vertical).unwrap()),
-            r: Some(RsvgLength::parse_str("50%", LengthDir::Both).unwrap()),
+            cx: Some(Length::parse_str("50%", LengthDir::Horizontal).unwrap()),
+            cy: Some(Length::parse_str("50%", LengthDir::Vertical).unwrap()),
+            r: Some(Length::parse_str("50%", LengthDir::Both).unwrap()),
 
             fx: None,
             fy: None,
@@ -437,8 +436,7 @@ fn resolve_gradient(gradient: &Gradient, draw_ctx: &mut DrawingCtx) -> Gradient
                     result.resolve_from_fallback(&fallback_grad)
                 });
                 Some(())
-            })
-            .or_else(|| {
+            }).or_else(|| {
                 result.resolve_from_defaults();
                 Some(())
             });
