@@ -1,16 +1,12 @@
 use cssparser::{Parser, Token};
-use libc;
 use std::f64::consts::*;
 
-use drawing_ctx::{DrawingCtx, RsvgDrawingCtx};
+use drawing_ctx::DrawingCtx;
 use error::*;
 use parsers::Parse;
 use parsers::ParseError;
-use state::{ComputedValues, RsvgComputedValues};
-use util::utf8_cstr;
+use state::ComputedValues;
 
-// Keep this in sync with ../../rsvg-private.h:LengthUnit
-#[repr(C)]
 #[derive(Debug, PartialEq, Copy, Clone)]
 pub enum LengthUnit {
     Default,
@@ -20,8 +16,6 @@ pub enum LengthUnit {
     Inch,
 }
 
-// Keep this in sync with ../../rsvg-private.h:LengthDir
-#[repr(C)]
 #[derive(Debug, PartialEq, Copy, Clone)]
 pub enum LengthDir {
     Horizontal,
@@ -29,22 +23,16 @@ pub enum LengthDir {
     Both,
 }
 
-// This is *not* an opaque struct; it is actually visible to the C code.  It is so
-// that the remaining C code can create RsvgLength values as part of existing
-// structures or objects, without allocations on the heap.
-//
-// Keep this in sync with ../../rsvg-private.h:RsvgLength
-#[repr(C)]
 #[derive(Debug, PartialEq, Copy, Clone)]
-pub struct RsvgLength {
+pub struct Length {
     pub length: f64,
     pub unit: LengthUnit,
     dir: LengthDir,
 }
 
-impl Default for RsvgLength {
-    fn default() -> RsvgLength {
-        RsvgLength {
+impl Default for Length {
+    fn default() -> Length {
+        Length {
             length: 0.0,
             unit: LengthUnit::Default,
             dir: LengthDir::Both,
@@ -57,14 +45,6 @@ const CM_PER_INCH: f64 = 2.54;
 const MM_PER_INCH: f64 = 25.4;
 const PICA_PER_INCH: f64 = 6.0;
 
-#[no_mangle]
-pub extern "C" fn rsvg_length_parse(string: *const libc::c_char, dir: LengthDir) -> RsvgLength {
-    let my_string = unsafe { utf8_cstr(string) };
-
-    // FIXME: this ignores errors; propagate them upstream
-    RsvgLength::parse_str(my_string, dir).unwrap_or_else(|_| RsvgLength::default())
-}
-
 // https://www.w3.org/TR/SVG/types.html#DataTypeLength
 // https://www.w3.org/TR/2008/REC-CSS2-20080411/syndata.html#length-units
 // Lengths have units.  When they need to be need resolved to
@@ -72,7 +52,7 @@ pub extern "C" fn rsvg_length_parse(string: *const libc::c_char, dir: LengthDir)
 // need to know if they are horizontal/vertical/both.  For example,
 // a some_object.width="50%" is 50% with respect to the current
 // viewport's width.  In this case, the @dir argument is used
-// inside RsvgLength::normalize(), when it needs to know to what the
+// inside Length::normalize(), when it needs to know to what the
 // length refers.
 
 fn make_err() -> AttributeError {
@@ -82,12 +62,12 @@ fn make_err() -> AttributeError {
     ))
 }
 
-impl Parse for RsvgLength {
+impl Parse for Length {
     type Data = LengthDir;
     type Err = AttributeError;
 
-    fn parse(parser: &mut Parser, dir: LengthDir) -> Result<RsvgLength, AttributeError> {
-        let length = RsvgLength::from_cssparser(parser, dir)?;
+    fn parse(parser: &mut Parser, dir: LengthDir) -> Result<Length, AttributeError> {
+        let length = Length::from_cssparser(parser, dir)?;
 
         parser.expect_exhausted().map_err(|_| make_err())?;
 
@@ -95,16 +75,16 @@ impl Parse for RsvgLength {
     }
 }
 
-impl RsvgLength {
-    pub fn new(l: f64, unit: LengthUnit, dir: LengthDir) -> RsvgLength {
-        RsvgLength {
+impl Length {
+    pub fn new(l: f64, unit: LengthUnit, dir: LengthDir) -> Length {
+        Length {
             length: l,
             unit,
             dir,
         }
     }
 
-    pub fn check_nonnegative(self) -> Result<RsvgLength, AttributeError> {
+    pub fn check_nonnegative(self) -> Result<Length, AttributeError> {
         if self.length >= 0.0 {
             Ok(self)
         } else {
@@ -151,21 +131,14 @@ impl RsvgLength {
     ) -> f64 {
         match self.unit {
             LengthUnit::Default => self.length,
-
             LengthUnit::Percent => self.length * width_or_height,
-
             LengthUnit::FontEm => self.length * font_size,
-
             LengthUnit::FontEx => self.length * font_size / 2.0,
-
             LengthUnit::Inch => self.length * pixels_per_inch,
         }
     }
 
-    pub fn from_cssparser(
-        parser: &mut Parser,
-        dir: LengthDir,
-    ) -> Result<RsvgLength, AttributeError> {
+    pub fn from_cssparser(parser: &mut Parser, dir: LengthDir) -> Result<Length, AttributeError> {
         let length = {
             let token = parser.next().map_err(|_| {
                 AttributeError::Parse(ParseError::new(
@@ -174,13 +147,13 @@ impl RsvgLength {
             })?;
 
             match *token {
-                Token::Number { value, .. } => RsvgLength {
+                Token::Number { value, .. } => Length {
                     length: f64::from(value),
                     unit: LengthUnit::Default,
                     dir,
                 },
 
-                Token::Percentage { unit_value, .. } => RsvgLength {
+                Token::Percentage { unit_value, .. } => Length {
                     length: f64::from(unit_value),
                     unit: LengthUnit::Percent,
                     dir,
@@ -192,49 +165,49 @@ impl RsvgLength {
                     let value = f64::from(value);
 
                     match unit.as_ref() {
-                        "em" => RsvgLength {
+                        "em" => Length {
                             length: value,
                             unit: LengthUnit::FontEm,
                             dir,
                         },
 
-                        "ex" => RsvgLength {
+                        "ex" => Length {
                             length: value,
                             unit: LengthUnit::FontEx,
                             dir,
                         },
 
-                        "pt" => RsvgLength {
+                        "pt" => Length {
                             length: value / POINTS_PER_INCH,
                             unit: LengthUnit::Inch,
                             dir,
                         },
 
-                        "in" => RsvgLength {
+                        "in" => Length {
                             length: value,
                             unit: LengthUnit::Inch,
                             dir,
                         },
 
-                        "cm" => RsvgLength {
+                        "cm" => Length {
                             length: value / CM_PER_INCH,
                             unit: LengthUnit::Inch,
                             dir,
                         },
 
-                        "mm" => RsvgLength {
+                        "mm" => Length {
                             length: value / MM_PER_INCH,
                             unit: LengthUnit::Inch,
                             dir,
                         },
 
-                        "pc" => RsvgLength {
+                        "pc" => Length {
                             length: value / PICA_PER_INCH,
                             unit: LengthUnit::Inch,
                             dir,
                         },
 
-                        "px" => RsvgLength {
+                        "px" => Length {
                             length: value,
                             unit: LengthUnit::Default,
                             dir,
@@ -295,11 +268,11 @@ pub enum FontSizeSpec {
     Large,
     XLarge,
     XXLarge,
-    Value(RsvgLength),
+    Value(Length),
 }
 
 impl FontSizeSpec {
-    pub fn value(&self) -> RsvgLength {
+    pub fn value(&self) -> Length {
         match self {
             FontSizeSpec::Value(s) => s.clone(),
             _ => unreachable!(),
@@ -312,31 +285,31 @@ impl FontSizeSpec {
         let size = v.font_size.0.value();
 
         let new_size = match self {
-            FontSizeSpec::Smaller => RsvgLength::new(size.length / 1.2, size.unit, LengthDir::Both),
-            FontSizeSpec::Larger => RsvgLength::new(size.length * 1.2, size.unit, LengthDir::Both),
+            FontSizeSpec::Smaller => Length::new(size.length / 1.2, size.unit, LengthDir::Both),
+            FontSizeSpec::Larger => Length::new(size.length * 1.2, size.unit, LengthDir::Both),
             FontSizeSpec::XXSmall => {
-                RsvgLength::new(compute_points(-3.0), LengthUnit::Inch, LengthDir::Both)
+                Length::new(compute_points(-3.0), LengthUnit::Inch, LengthDir::Both)
             }
             FontSizeSpec::XSmall => {
-                RsvgLength::new(compute_points(-2.0), LengthUnit::Inch, LengthDir::Both)
+                Length::new(compute_points(-2.0), LengthUnit::Inch, LengthDir::Both)
             }
             FontSizeSpec::Small => {
-                RsvgLength::new(compute_points(-1.0), LengthUnit::Inch, LengthDir::Both)
+                Length::new(compute_points(-1.0), LengthUnit::Inch, LengthDir::Both)
             }
             FontSizeSpec::Medium => {
-                RsvgLength::new(compute_points(0.0), LengthUnit::Inch, LengthDir::Both)
+                Length::new(compute_points(0.0), LengthUnit::Inch, LengthDir::Both)
             }
             FontSizeSpec::Large => {
-                RsvgLength::new(compute_points(1.0), LengthUnit::Inch, LengthDir::Both)
+                Length::new(compute_points(1.0), LengthUnit::Inch, LengthDir::Both)
             }
             FontSizeSpec::XLarge => {
-                RsvgLength::new(compute_points(2.0), LengthUnit::Inch, LengthDir::Both)
+                Length::new(compute_points(2.0), LengthUnit::Inch, LengthDir::Both)
             }
             FontSizeSpec::XXLarge => {
-                RsvgLength::new(compute_points(3.0), LengthUnit::Inch, LengthDir::Both)
+                Length::new(compute_points(3.0), LengthUnit::Inch, LengthDir::Both)
             }
             FontSizeSpec::Value(s) if s.unit == LengthUnit::Percent => {
-                RsvgLength::new(size.length * s.length, size.unit, LengthDir::Both)
+                Length::new(size.length * s.length, size.unit, LengthDir::Both)
             }
             FontSizeSpec::Value(s) => s.clone(),
         };
@@ -356,7 +329,7 @@ impl Parse for FontSizeSpec {
     fn parse(parser: &mut Parser, _: Self::Data) -> Result<FontSizeSpec, ::error::AttributeError> {
         let parser_state = parser.state();
 
-        RsvgLength::parse(parser, LengthDir::Both)
+        Length::parse(parser, LengthDir::Both)
             .and_then(|s| Ok(FontSizeSpec::Value(s)))
             .or_else(|e| {
                 parser.reset(&parser_state);
@@ -392,7 +365,7 @@ impl Parse for FontSizeSpec {
 #[derive(Debug, PartialEq, Clone)]
 pub enum Dasharray {
     None,
-    Array(Vec<RsvgLength>),
+    Array(Vec<Length>),
 }
 
 impl Default for Dasharray {
@@ -415,13 +388,12 @@ impl Parse for Dasharray {
 }
 
 // This does not handle "inherit" or "none" state, the caller is responsible for that.
-fn parse_dash_array(parser: &mut Parser) -> Result<Vec<RsvgLength>, AttributeError> {
+fn parse_dash_array(parser: &mut Parser) -> Result<Vec<Length>, AttributeError> {
     let mut dasharray = Vec::new();
 
     loop {
         dasharray.push(
-            RsvgLength::from_cssparser(parser, LengthDir::Both)
-                .and_then(RsvgLength::check_nonnegative)?,
+            Length::from_cssparser(parser, LengthDir::Both).and_then(Length::check_nonnegative)?,
         );
 
         if parser.is_exhausted() {
@@ -434,38 +406,6 @@ fn parse_dash_array(parser: &mut Parser) -> Result<Vec<RsvgLength>, AttributeErr
     Ok(dasharray)
 }
 
-#[no_mangle]
-pub extern "C" fn rsvg_length_normalize(
-    raw_length: *const RsvgLength,
-    values: RsvgComputedValues,
-    draw_ctx: *const RsvgDrawingCtx,
-) -> f64 {
-    assert!(!raw_length.is_null());
-    let length: &RsvgLength = unsafe { &*raw_length };
-
-    assert!(!values.is_null());
-    let values = unsafe { &*values };
-
-    assert!(!draw_ctx.is_null());
-    let draw_ctx = unsafe { &*(draw_ctx as *const DrawingCtx) };
-
-    length.normalize(values, draw_ctx)
-}
-
-#[no_mangle]
-pub extern "C" fn rsvg_length_hand_normalize(
-    raw_length: *const RsvgLength,
-    pixels_per_inch: f64,
-    width_or_height: f64,
-    font_size: f64,
-) -> f64 {
-    assert!(!raw_length.is_null());
-
-    let length: &RsvgLength = unsafe { &*raw_length };
-
-    length.hand_normalize(pixels_per_inch, width_or_height, font_size)
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -473,8 +413,8 @@ mod tests {
     #[test]
     fn parses_default() {
         assert_eq!(
-            RsvgLength::parse_str("42", LengthDir::Horizontal),
-            Ok(RsvgLength::new(
+            Length::parse_str("42", LengthDir::Horizontal),
+            Ok(Length::new(
                 42.0,
                 LengthUnit::Default,
                 LengthDir::Horizontal
@@ -482,8 +422,8 @@ mod tests {
         );
 
         assert_eq!(
-            RsvgLength::parse_str("-42px", LengthDir::Horizontal),
-            Ok(RsvgLength::new(
+            Length::parse_str("-42px", LengthDir::Horizontal),
+            Ok(Length::new(
                 -42.0,
                 LengthUnit::Default,
                 LengthDir::Horizontal
@@ -494,75 +434,63 @@ mod tests {
     #[test]
     fn parses_percent() {
         assert_eq!(
-            RsvgLength::parse_str("50.0%", LengthDir::Horizontal),
-            Ok(RsvgLength::new(
-                0.5,
-                LengthUnit::Percent,
-                LengthDir::Horizontal
-            ))
+            Length::parse_str("50.0%", LengthDir::Horizontal),
+            Ok(Length::new(0.5, LengthUnit::Percent, LengthDir::Horizontal))
         );
     }
 
     #[test]
     fn parses_font_em() {
         assert_eq!(
-            RsvgLength::parse_str("22.5em", LengthDir::Vertical),
-            Ok(RsvgLength::new(
-                22.5,
-                LengthUnit::FontEm,
-                LengthDir::Vertical
-            ))
+            Length::parse_str("22.5em", LengthDir::Vertical),
+            Ok(Length::new(22.5, LengthUnit::FontEm, LengthDir::Vertical))
         );
     }
 
     #[test]
     fn parses_font_ex() {
         assert_eq!(
-            RsvgLength::parse_str("22.5ex", LengthDir::Vertical),
-            Ok(RsvgLength::new(
-                22.5,
-                LengthUnit::FontEx,
-                LengthDir::Vertical
-            ))
+            Length::parse_str("22.5ex", LengthDir::Vertical),
+            Ok(Length::new(22.5, LengthUnit::FontEx, LengthDir::Vertical))
         );
     }
 
     #[test]
     fn parses_physical_units() {
         assert_eq!(
-            RsvgLength::parse_str("72pt", LengthDir::Both),
-            Ok(RsvgLength::new(1.0, LengthUnit::Inch, LengthDir::Both))
+            Length::parse_str("72pt", LengthDir::Both),
+            Ok(Length::new(1.0, LengthUnit::Inch, LengthDir::Both))
         );
 
         assert_eq!(
-            RsvgLength::parse_str("-22.5in", LengthDir::Both),
-            Ok(RsvgLength::new(-22.5, LengthUnit::Inch, LengthDir::Both))
+            Length::parse_str("-22.5in", LengthDir::Both),
+            Ok(Length::new(-22.5, LengthUnit::Inch, LengthDir::Both))
         );
 
         assert_eq!(
-            RsvgLength::parse_str("-254cm", LengthDir::Both),
-            Ok(RsvgLength::new(-100.0, LengthUnit::Inch, LengthDir::Both))
+            Length::parse_str("-254cm", LengthDir::Both),
+            Ok(Length::new(-100.0, LengthUnit::Inch, LengthDir::Both))
         );
 
         assert_eq!(
-            RsvgLength::parse_str("254mm", LengthDir::Both),
-            Ok(RsvgLength::new(10.0, LengthUnit::Inch, LengthDir::Both))
+            Length::parse_str("254mm", LengthDir::Both),
+            Ok(Length::new(10.0, LengthUnit::Inch, LengthDir::Both))
         );
 
         assert_eq!(
-            RsvgLength::parse_str("60pc", LengthDir::Both),
-            Ok(RsvgLength::new(10.0, LengthUnit::Inch, LengthDir::Both))
+            Length::parse_str("60pc", LengthDir::Both),
+            Ok(Length::new(10.0, LengthUnit::Inch, LengthDir::Both))
         );
     }
 
     #[test]
     fn empty_length_yields_error() {
-        assert!(is_parse_error(&RsvgLength::parse_str("", LengthDir::Both)));
+        assert!(is_parse_error(&Length::parse_str("", LengthDir::Both)));
     }
 
     #[test]
     fn invalid_unit_yields_error() {
-        assert!(is_parse_error(&RsvgLength::parse_str(
+        assert!(is_parse_error(&Length::parse_str(
             "8furlong",
             LengthDir::Both
         )));
@@ -571,12 +499,12 @@ mod tests {
     #[test]
     fn check_nonnegative_works() {
         assert!(
-            RsvgLength::parse_str("0", LengthDir::Both)
+            Length::parse_str("0", LengthDir::Both)
                 .and_then(|l| l.check_nonnegative())
                 .is_ok()
         );
         assert!(
-            RsvgLength::parse_str("-10", LengthDir::Both)
+            Length::parse_str("-10", LengthDir::Both)
                 .and_then(|l| l.check_nonnegative())
                 .is_err()
         );
@@ -594,7 +522,7 @@ mod tests {
     #[test]
     fn parses_dash_array() {
         // helper to cut down boilderplate
-        let length_parse = |s| RsvgLength::parse_str(s, LengthDir::Both).unwrap();
+        let length_parse = |s| Length::parse_str(s, LengthDir::Both).unwrap();
 
         let expected = Dasharray::Array(vec![
             length_parse("1"),
