@@ -10,6 +10,7 @@ extern crate glib;
 extern crate glib_sys;
 extern crate itertools;
 extern crate libc;
+extern crate nalgebra;
 extern crate owning_ref;
 extern crate pango;
 extern crate pango_cairo_sys;
@@ -23,13 +24,12 @@ extern crate lazy_static;
 #[macro_use]
 extern crate downcast_rs;
 
-#[macro_use]
-extern crate rulinalg;
-
 pub use attributes::rsvg_attribute_from_name;
 
 pub use color::{rsvg_css_parse_color, ColorKind, ColorSpec};
 
+pub use defs::{rsvg_defs_free, rsvg_defs_lookup, rsvg_defs_new};
+
 pub use drawing_ctx::{
     rsvg_drawing_ctx_add_node_and_ancestors_to_stack,
     rsvg_drawing_ctx_draw_node_from_stack,
@@ -38,15 +38,6 @@ pub use drawing_ctx::{
     rsvg_drawing_ctx_new,
 };
 
-pub use length::{
-    rsvg_length_hand_normalize,
-    rsvg_length_normalize,
-    rsvg_length_parse,
-    LengthDir,
-    LengthUnit,
-    RsvgLength,
-};
-
 pub use load::{rsvg_load_new_node, rsvg_load_set_node_atts};
 
 pub use node::{
@@ -54,12 +45,11 @@ pub use node::{
     rsvg_node_children_iter_begin,
     rsvg_node_children_iter_end,
     rsvg_node_children_iter_next,
-    rsvg_node_children_iter_next_back,
+    rsvg_node_find_last_chars_child,
     rsvg_node_get_parent,
     rsvg_node_get_type,
     rsvg_node_is_same,
     rsvg_node_ref,
-    rsvg_node_register_in_defs,
     rsvg_node_set_attribute_parse_error,
     rsvg_node_unref,
     rsvg_root_node_cascade,
@@ -75,14 +65,9 @@ pub use property_bag::{
     rsvg_property_bag_new,
 };
 
-pub use state::{
-    rsvg_computed_values_get_lighting_color_argb,
-    rsvg_state_free,
-    rsvg_state_new,
-    rsvg_state_parse_style_pair,
-};
+pub use state::{rsvg_state_free, rsvg_state_new, rsvg_state_parse_style_pair};
 
-pub use structure::{rsvg_node_svg_apply_atts, rsvg_node_svg_get_size, rsvg_node_svg_get_view_box};
+pub use structure::{rsvg_node_svg_apply_atts, rsvg_node_svg_get_size};
 
 pub use text::{rsvg_node_chars_append, rsvg_node_chars_new};
 
