@@ -12,7 +12,6 @@ use std::str::FromStr;
 
 use attributes::Attribute;
 use cond::{RequiredExtensions, RequiredFeatures, SystemLanguage};
-use defs::{self, RsvgDefs};
 use drawing_ctx::DrawingCtx;
 use error::*;
 use handle::RsvgHandle;
@@ -123,6 +122,10 @@ pub trait NodeTrait: Downcast {
     /// from defaults in the node's `State`.
     fn set_overridden_properties(&self, _state: &mut State) {}
 
+    fn accept_chars(&self) -> bool {
+        false
+    }
+
     fn draw(
         &self,
         _node: &RsvgNode,
@@ -279,12 +282,6 @@ impl Node {
         self.class.as_ref().map(String::as_str)
     }
 
-    pub fn register(&self, node: &RsvgNode, defs: *mut RsvgDefs) {
-        if let Some(ref id) = self.id {
-            defs::register_node_by_id(defs, id, node);
-        }
-    }
-
     pub fn get_transform(&self) -> Matrix {
         self.transform.get()
     }
@@ -492,6 +489,30 @@ impl Node {
         let state = self.get_state_mut();
         state.values.overflow = SpecifiedValue::Specified(Overflow::Hidden);
     }
+
+    pub fn accept_chars(&self) -> bool {
+        self.node_impl.accept_chars()
+    }
+
+    // find the last Chars node so that we can coalesce
+    // the text and avoid screwing up the Pango layouts
+    pub fn find_last_chars_child(&self) -> Option<Rc<Node>> {
+        for child in self.children().rev() {
+            match child.get_type() {
+                NodeType::Chars => return Some(child),
+
+                // If a node that accepts chars is encountered before
+                // any chars node (which means for instance that there
+                // is a tspan node after any chars nodes, because this
+                // is backwards iteration), return None.
+                _ if child.accept_chars() => return None,
+
+                _ => {}
+            }
+        }
+
+        None
+    }
 }
 
 // Sigh, rsvg_state_free() is only available if we are being linked into
@@ -658,16 +679,6 @@ pub extern "C" fn rsvg_node_add_child(raw_node: *mut RsvgNode, raw_child: *const
     node.add_child(child);
 }
 
-#[no_mangle]
-pub extern "C" fn rsvg_node_register_in_defs(raw_node: *const RsvgNode, defs: *mut RsvgDefs) {
-    assert!(!raw_node.is_null());
-    assert!(!defs.is_null());
-
-    let node: &RsvgNode = unsafe { &*raw_node };
-
-    node.register(node, defs);
-}
-
 #[no_mangle]
 pub extern "C" fn rsvg_node_set_attribute_parse_error(
     raw_node: *const RsvgNode,
@@ -691,6 +702,30 @@ pub extern "C" fn rsvg_node_set_attribute_parse_error(
     }
 }
 
+#[no_mangle]
+pub extern "C" fn rsvg_node_find_last_chars_child(
+    raw_node: *const RsvgNode,
+    out_accept_chars: *mut glib_sys::gboolean,
+) -> *mut RsvgNode {
+    assert!(!raw_node.is_null());
+    let node: &RsvgNode = unsafe { &*raw_node };
+
+    let accept_chars = node.accept_chars();
+
+    assert!(!out_accept_chars.is_null());
+    unsafe {
+        *out_accept_chars = accept_chars.to_glib();
+    }
+
+    if accept_chars {
+        if let Some(chars) = node.find_last_chars_child() {
+            return box_node(chars);
+        }
+    }
+
+    ptr::null_mut()
+}
+
 #[no_mangle]
 pub extern "C" fn rsvg_node_children_iter_begin(raw_node: *const RsvgNode) -> *mut Children {
     assert!(!raw_node.is_null());
@@ -727,27 +762,6 @@ pub extern "C" fn rsvg_node_children_iter_next(
     }
 }
 
-#[no_mangle]
-pub extern "C" fn rsvg_node_children_iter_next_back(
-    iter: *mut Children,
-    out_child: *mut *mut RsvgNode,
-) -> glib_sys::gboolean {
-    assert!(!iter.is_null());
-
-    let iter = unsafe { &mut *iter };
-    if let Some(child) = iter.next_back() {
-        unsafe {
-            *out_child = box_node(child);
-        }
-        true.to_glib()
-    } else {
-        unsafe {
-            *out_child = ptr::null_mut();
-        }
-        false.to_glib()
-    }
-}
-
 #[no_mangle]
 pub extern "C" fn rsvg_root_node_cascade(raw_node: *const RsvgNode) {
     assert!(!raw_node.is_null());
@@ -977,15 +991,5 @@ mod tests {
         assert_eq!(result, true);
         assert!(Rc::ptr_eq(unsafe { &*c }, &child));
         rsvg_node_unref(c);
-
-        let result: bool = from_glib(rsvg_node_children_iter_next_back(iter, &mut c));
-        assert_eq!(result, true);
-        assert!(Rc::ptr_eq(unsafe { &*c }, &second_child));
-        rsvg_node_unref(c);
-
-        let result: bool = from_glib(rsvg_node_children_iter_next(iter, &mut c));
-        assert_eq!(result, false);
-        let result: bool = from_glib(rsvg_node_children_iter_next_back(iter, &mut c));
-        assert_eq!(result, false);
     }
 }
