@@ -1,7 +1,7 @@
-use cssparser;
+use cssparser::{self, Parser};
 
 use bbox::BoundingBox;
-use drawing_ctx;
+use drawing_ctx::DrawingCtx;
 use error::*;
 use gradient;
 use node::NodeType;
@@ -23,10 +23,7 @@ impl Parse for PaintServer {
     type Data = ();
     type Err = AttributeError;
 
-    fn parse(s: &str, _: ()) -> Result<PaintServer, AttributeError> {
-        let mut input = cssparser::ParserInput::new(s);
-        let mut parser = cssparser::Parser::new(&mut input);
-
+    fn parse(parser: &mut Parser, _: ()) -> Result<PaintServer, AttributeError> {
         if parser.try(|i| i.expect_ident_matching("none")).is_ok() {
             Ok(PaintServer::None)
         } else if let Ok(url) = parser.try(|i| i.expect_url()) {
@@ -45,7 +42,7 @@ impl Parse for PaintServer {
                 alternate,
             })
         } else {
-            cssparser::Color::parse(&mut parser)
+            cssparser::Color::parse(parser)
                 .map(PaintServer::SolidColor)
                 .map_err(AttributeError::from)
         }
@@ -53,7 +50,7 @@ impl Parse for PaintServer {
 }
 
 fn set_color(
-    ctx: *mut drawing_ctx::RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     color: &cssparser::Color,
     opacity: &UnitInterval,
     current_color: &cssparser::RGBA,
@@ -64,7 +61,7 @@ fn set_color(
     };
 
     let &UnitInterval(o) = opacity;
-    drawing_ctx::get_cairo_context(ctx).set_source_rgba(
+    draw_ctx.get_cairo_context().set_source_rgba(
         f64::from(rgba.red_f32()),
         f64::from(rgba.green_f32()),
         f64::from(rgba.blue_f32()),
@@ -73,7 +70,7 @@ fn set_color(
 }
 
 pub fn set_source_paint_server(
-    c_ctx: *mut drawing_ctx::RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     ps: &PaintServer,
     opacity: &UnitInterval,
     bbox: &BoundingBox,
@@ -88,29 +85,34 @@ pub fn set_source_paint_server(
         } => {
             had_paint_server = false;
 
-            if let Some(acquired) = drawing_ctx::get_acquired_node(c_ctx, iri.as_str()) {
+            if let Some(acquired) = draw_ctx.get_acquired_node(iri.as_str()) {
                 let node = acquired.get();
 
                 if node.get_type() == NodeType::LinearGradient
                     || node.get_type() == NodeType::RadialGradient
                 {
                     had_paint_server = gradient::gradient_resolve_fallbacks_and_set_pattern(
-                        &node, c_ctx, opacity, bbox,
+                        &node, draw_ctx, opacity, bbox,
                     );
                 } else if node.get_type() == NodeType::Pattern {
                     had_paint_server =
-                        pattern::pattern_resolve_fallbacks_and_set_pattern(&node, c_ctx, bbox);
+                        pattern::pattern_resolve_fallbacks_and_set_pattern(&node, draw_ctx, bbox);
                 }
             }
 
             if !had_paint_server && alternate.is_some() {
-                set_color(c_ctx, alternate.as_ref().unwrap(), opacity, current_color);
+                set_color(
+                    draw_ctx,
+                    alternate.as_ref().unwrap(),
+                    opacity,
+                    current_color,
+                );
                 had_paint_server = true;
             }
         }
 
         PaintServer::SolidColor(color) => {
-            set_color(c_ctx, &color, opacity, current_color);
+            set_color(draw_ctx, &color, opacity, current_color);
             had_paint_server = true;
         }
 
@@ -128,27 +130,27 @@ mod tests {
 
     #[test]
     fn catches_invalid_syntax() {
-        assert!(PaintServer::parse("", ()).is_err());
-        assert!(PaintServer::parse("42", ()).is_err());
-        assert!(PaintServer::parse("invalid", ()).is_err());
+        assert!(PaintServer::parse_str("", ()).is_err());
+        assert!(PaintServer::parse_str("42", ()).is_err());
+        assert!(PaintServer::parse_str("invalid", ()).is_err());
     }
 
     #[test]
     fn parses_none() {
-        assert_eq!(PaintServer::parse("none", ()), Ok(PaintServer::None));
+        assert_eq!(PaintServer::parse_str("none", ()), Ok(PaintServer::None));
     }
 
     #[test]
     fn parses_solid_color() {
         assert_eq!(
-            PaintServer::parse("rgb(255, 128, 64, 0.5)", ()),
+            PaintServer::parse_str("rgb(255, 128, 64, 0.5)", ()),
             Ok(PaintServer::SolidColor(cssparser::Color::RGBA(
                 cssparser::RGBA::new(255, 128, 64, 128)
             )))
         );
 
         assert_eq!(
-            PaintServer::parse("currentColor", ()),
+            PaintServer::parse_str("currentColor", ()),
             Ok(PaintServer::SolidColor(cssparser::Color::CurrentColor))
         );
     }
@@ -156,7 +158,7 @@ mod tests {
     #[test]
     fn parses_iri() {
         assert_eq!(
-            PaintServer::parse("url(#link)", ()),
+            PaintServer::parse_str("url(#link)", ()),
             Ok(PaintServer::Iri {
                 iri: "#link".to_string(),
                 alternate: None,
@@ -164,7 +166,7 @@ mod tests {
         );
 
         assert_eq!(
-            PaintServer::parse("url(#link) none", ()),
+            PaintServer::parse_str("url(#link) none", ()),
             Ok(PaintServer::Iri {
                 iri: "#link".to_string(),
                 alternate: None,
@@ -172,7 +174,7 @@ mod tests {
         );
 
         assert_eq!(
-            PaintServer::parse("url(#link) #ff8040", ()),
+            PaintServer::parse_str("url(#link) #ff8040", ()),
             Ok(PaintServer::Iri {
                 iri: "#link".to_string(),
                 alternate: Some(cssparser::Color::RGBA(cssparser::RGBA::new(
@@ -182,7 +184,7 @@ mod tests {
         );
 
         assert_eq!(
-            PaintServer::parse("url(#link) rgb(255, 128, 64, 0.5)", ()),
+            PaintServer::parse_str("url(#link) rgb(255, 128, 64, 0.5)", ()),
             Ok(PaintServer::Iri {
                 iri: "#link".to_string(),
                 alternate: Some(cssparser::Color::RGBA(cssparser::RGBA::new(
@@ -192,13 +194,13 @@ mod tests {
         );
 
         assert_eq!(
-            PaintServer::parse("url(#link) currentColor", ()),
+            PaintServer::parse_str("url(#link) currentColor", ()),
             Ok(PaintServer::Iri {
                 iri: "#link".to_string(),
                 alternate: Some(cssparser::Color::CurrentColor),
             },)
         );
 
-        assert!(PaintServer::parse("url(#link) invalid", ()).is_err());
+        assert!(PaintServer::parse_str("url(#link) invalid", ()).is_err());
     }
 }
