@@ -231,19 +231,19 @@ impl NodeTrait for NodePoly {
 }
 
 pub struct NodeLine {
-    x1: Cell<RsvgLength>,
-    y1: Cell<RsvgLength>,
-    x2: Cell<RsvgLength>,
-    y2: Cell<RsvgLength>,
+    x1: Cell<Length>,
+    y1: Cell<Length>,
+    x2: Cell<Length>,
+    y2: Cell<Length>,
 }
 
 impl NodeLine {
     pub fn new() -> NodeLine {
         NodeLine {
-            x1: Cell::new(RsvgLength::default()),
-            y1: Cell::new(RsvgLength::default()),
-            x2: Cell::new(RsvgLength::default()),
-            y2: Cell::new(RsvgLength::default()),
+            x1: Cell::new(Length::default()),
+            y1: Cell::new(Length::default()),
+            x2: Cell::new(Length::default()),
+            y2: Cell::new(Length::default()),
         }
     }
 }
@@ -288,23 +288,23 @@ impl NodeTrait for NodeLine {
 
 pub struct NodeRect {
     // x, y, width, height
-    x: Cell<RsvgLength>,
-    y: Cell<RsvgLength>,
-    w: Cell<RsvgLength>,
-    h: Cell<RsvgLength>,
+    x: Cell<Length>,
+    y: Cell<Length>,
+    w: Cell<Length>,
+    h: Cell<Length>,
 
     // Radiuses for rounded corners
-    rx: Cell<Option<RsvgLength>>,
-    ry: Cell<Option<RsvgLength>>,
+    rx: Cell<Option<Length>>,
+    ry: Cell<Option<Length>>,
 }
 
 impl NodeRect {
     pub fn new() -> NodeRect {
         NodeRect {
-            x: Cell::new(RsvgLength::default()),
-            y: Cell::new(RsvgLength::default()),
-            w: Cell::new(RsvgLength::default()),
-            h: Cell::new(RsvgLength::default()),
+            x: Cell::new(Length::default()),
+            y: Cell::new(Length::default()),
+            w: Cell::new(Length::default()),
+            h: Cell::new(Length::default()),
 
             rx: Cell::new(None),
             ry: Cell::new(None),
@@ -322,13 +322,13 @@ impl NodeTrait for NodeRect {
                     "width",
                     value,
                     LengthDir::Horizontal,
-                    RsvgLength::check_nonnegative,
+                    Length::check_nonnegative,
                 )?),
                 Attribute::Height => self.h.set(parse_and_validate(
                     "height",
                     value,
                     LengthDir::Vertical,
-                    RsvgLength::check_nonnegative,
+                    Length::check_nonnegative,
                 )?),
 
                 Attribute::Rx => self.rx.set(
@@ -336,16 +336,12 @@ impl NodeTrait for NodeRect {
                         "rx",
                         value,
                         LengthDir::Horizontal,
-                        RsvgLength::check_nonnegative,
+                        Length::check_nonnegative,
                     ).map(Some)?,
                 ),
                 Attribute::Ry => self.ry.set(
-                    parse_and_validate(
-                        "ry",
-                        value,
-                        LengthDir::Vertical,
-                        RsvgLength::check_nonnegative,
-                    ).map(Some)?,
+                    parse_and_validate("ry", value, LengthDir::Vertical, Length::check_nonnegative)
+                        .map(Some)?,
                 ),
 
                 _ => (),
@@ -503,17 +499,17 @@ impl NodeTrait for NodeRect {
 }
 
 pub struct NodeCircle {
-    cx: Cell<RsvgLength>,
-    cy: Cell<RsvgLength>,
-    r: Cell<RsvgLength>,
+    cx: Cell<Length>,
+    cy: Cell<Length>,
+    r: Cell<Length>,
 }
 
 impl NodeCircle {
     pub fn new() -> NodeCircle {
         NodeCircle {
-            cx: Cell::new(RsvgLength::default()),
-            cy: Cell::new(RsvgLength::default()),
-            r: Cell::new(RsvgLength::default()),
+            cx: Cell::new(Length::default()),
+            cy: Cell::new(Length::default()),
+            r: Cell::new(Length::default()),
         }
     }
 }
@@ -528,7 +524,7 @@ impl NodeTrait for NodeCircle {
                     "r",
                     value,
                     LengthDir::Both,
-                    RsvgLength::check_nonnegative,
+                    Length::check_nonnegative,
                 )?),
 
                 _ => (),
@@ -556,19 +552,19 @@ impl NodeTrait for NodeCircle {
 }
 
 pub struct NodeEllipse {
-    cx: Cell<RsvgLength>,
-    cy: Cell<RsvgLength>,
-    rx: Cell<RsvgLength>,
-    ry: Cell<RsvgLength>,
+    cx: Cell<Length>,
+    cy: Cell<Length>,
+    rx: Cell<Length>,
+    ry: Cell<Length>,
 }
 
 impl NodeEllipse {
     pub fn new() -> NodeEllipse {
         NodeEllipse {
-            cx: Cell::new(RsvgLength::default()),
-            cy: Cell::new(RsvgLength::default()),
-            rx: Cell::new(RsvgLength::default()),
-            ry: Cell::new(RsvgLength::default()),
+            cx: Cell::new(Length::default()),
+            cy: Cell::new(Length::default()),
+            rx: Cell::new(Length::default()),
+            ry: Cell::new(Length::default()),
         }
     }
 }
@@ -584,13 +580,13 @@ impl NodeTrait for NodeEllipse {
                     "rx",
                     value,
                     LengthDir::Horizontal,
-                    RsvgLength::check_nonnegative,
+                    Length::check_nonnegative,
                 )?),
                 Attribute::Ry => self.ry.set(parse_and_validate(
                     "ry",
                     value,
                     LengthDir::Vertical,
-                    RsvgLength::check_nonnegative,
+                    Length::check_nonnegative,
                 )?),
 
                 _ => (),
