@@ -1,8 +1,6 @@
 use std::cell::Cell;
 use std::cell::RefCell;
 
-use cairo;
-use cairo_sys;
 use glib::translate::*;
 use glib_sys;
 
@@ -12,6 +10,7 @@ use drawing_ctx::DrawingCtx;
 use float_eq_cairo::ApproxEqCairo;
 use handle::RsvgHandle;
 use length::*;
+use libc;
 use node::*;
 use parsers::{parse, parse_and_validate, Parse};
 use property_bag::{OwnedPropertyBag, PropertyBag};
@@ -93,10 +92,10 @@ impl NodeTrait for NodeSwitch {
 
 pub struct NodeSvg {
     preserve_aspect_ratio: Cell<AspectRatio>,
-    x: Cell<RsvgLength>,
-    y: Cell<RsvgLength>,
-    w: Cell<RsvgLength>,
-    h: Cell<RsvgLength>,
+    x: Cell<Length>,
+    y: Cell<Length>,
+    w: Cell<Length>,
+    h: Cell<Length>,
     vbox: Cell<Option<ViewBox>>,
     pbag: RefCell<Option<OwnedPropertyBag>>,
 }
@@ -105,10 +104,10 @@ impl NodeSvg {
     pub fn new() -> NodeSvg {
         NodeSvg {
             preserve_aspect_ratio: Cell::new(AspectRatio::default()),
-            x: Cell::new(RsvgLength::parse_str("0", LengthDir::Horizontal).unwrap()),
-            y: Cell::new(RsvgLength::parse_str("0", LengthDir::Vertical).unwrap()),
-            w: Cell::new(RsvgLength::parse_str("100%", LengthDir::Horizontal).unwrap()),
-            h: Cell::new(RsvgLength::parse_str("100%", LengthDir::Vertical).unwrap()),
+            x: Cell::new(Length::parse_str("0", LengthDir::Horizontal).unwrap()),
+            y: Cell::new(Length::parse_str("0", LengthDir::Vertical).unwrap()),
+            w: Cell::new(Length::parse_str("100%", LengthDir::Horizontal).unwrap()),
+            h: Cell::new(Length::parse_str("100%", LengthDir::Vertical).unwrap()),
             vbox: Cell::new(None),
             pbag: RefCell::new(None),
         }
@@ -148,14 +147,14 @@ impl NodeTrait for NodeSvg {
                     "width",
                     value,
                     LengthDir::Horizontal,
-                    RsvgLength::check_nonnegative,
+                    Length::check_nonnegative,
                 )?),
 
                 Attribute::Height => self.h.set(parse_and_validate(
                     "height",
                     value,
                     LengthDir::Vertical,
-                    RsvgLength::check_nonnegative,
+                    Length::check_nonnegative,
                 )?),
 
                 Attribute::ViewBox => self.vbox.set(parse("viewBox", value, ()).map(Some)?),
@@ -211,18 +210,18 @@ impl NodeTrait for NodeSvg {
 
 pub struct NodeUse {
     link: RefCell<Option<String>>,
-    x: Cell<RsvgLength>,
-    y: Cell<RsvgLength>,
-    w: Cell<Option<RsvgLength>>,
-    h: Cell<Option<RsvgLength>>,
+    x: Cell<Length>,
+    y: Cell<Length>,
+    w: Cell<Option<Length>>,
+    h: Cell<Option<Length>>,
 }
 
 impl NodeUse {
     pub fn new() -> NodeUse {
         NodeUse {
             link: RefCell::new(None),
-            x: Cell::new(RsvgLength::default()),
-            y: Cell::new(RsvgLength::default()),
+            x: Cell::new(Length::default()),
+            y: Cell::new(Length::default()),
             w: Cell::new(None),
             h: Cell::new(None),
         }
@@ -243,7 +242,7 @@ impl NodeTrait for NodeUse {
                         "width",
                         value,
                         LengthDir::Horizontal,
-                        RsvgLength::check_nonnegative,
+                        Length::check_nonnegative,
                     ).map(Some)?,
                 ),
                 Attribute::Height => self.h.set(
@@ -251,7 +250,7 @@ impl NodeTrait for NodeUse {
                         "height",
                         value,
                         LengthDir::Vertical,
-                        RsvgLength::check_nonnegative,
+                        Length::check_nonnegative,
                     ).map(Some)?,
                 ),
 
@@ -301,12 +300,12 @@ impl NodeTrait for NodeUse {
         let nw = self
             .w
             .get()
-            .unwrap_or_else(|| RsvgLength::parse_str("100%", LengthDir::Horizontal).unwrap())
+            .unwrap_or_else(|| Length::parse_str("100%", LengthDir::Horizontal).unwrap())
             .normalize(values, draw_ctx);
         let nh = self
             .h
             .get()
-            .unwrap_or_else(|| RsvgLength::parse_str("100%", LengthDir::Vertical).unwrap())
+            .unwrap_or_else(|| Length::parse_str("100%", LengthDir::Vertical).unwrap())
             .normalize(values, draw_ctx);
 
         // width or height set to 0 disables rendering of the element
@@ -400,54 +399,36 @@ impl NodeTrait for NodeSymbol {
 #[no_mangle]
 pub extern "C" fn rsvg_node_svg_get_size(
     raw_node: *const RsvgNode,
-    out_width: *mut RsvgLength,
-    out_height: *mut RsvgLength,
-) {
+    dpi_x: libc::c_double,
+    dpi_y: libc::c_double,
+    out_width: *mut i32,
+    out_height: *mut i32,
+) -> glib_sys::gboolean {
     assert!(!raw_node.is_null());
     let node: &RsvgNode = unsafe { &*raw_node };
 
     assert!(!out_width.is_null());
     assert!(!out_height.is_null());
 
-    node.with_impl(|svg: &NodeSvg| unsafe {
-        *out_width = svg.w.get();
-        *out_height = svg.h.get();
-    });
-}
-
-#[no_mangle]
-pub extern "C" fn rsvg_node_svg_get_view_box(
-    raw_node: *const RsvgNode,
-    out_vbox: *mut cairo_sys::cairo_rectangle_t,
-) -> glib_sys::gboolean {
-    assert!(!raw_node.is_null());
-    assert!(!out_vbox.is_null());
-
-    let node: &RsvgNode = unsafe { &*raw_node };
-
-    let mut vbox: Option<ViewBox> = None;
-
-    node.with_impl(|svg: &NodeSvg| {
-        vbox = svg.vbox.get();
-    });
-
-    if let Some(vb) = vbox {
-        unsafe {
-            *out_vbox = vb.0;
-        }
-        true.to_glib()
-    } else {
-        unsafe {
-            *out_vbox = cairo::Rectangle {
-                x: 0.0,
-                y: 0.0,
-                width: 0.0,
-                height: 0.0,
-            };
-        }
-
-        false.to_glib()
-    }
+    node.with_impl(
+        |svg: &NodeSvg| match (svg.w.get(), svg.h.get(), svg.vbox.get()) {
+            (w, h, Some(vb)) => {
+                unsafe {
+                    *out_width = w.hand_normalize(dpi_x, vb.0.width, 12.0).round() as i32;
+                    *out_height = h.hand_normalize(dpi_y, vb.0.height, 12.0).round() as i32;
+                }
+                true.to_glib()
+            }
+            (w, h, None) if w.unit != LengthUnit::Percent && h.unit != LengthUnit::Percent => {
+                unsafe {
+                    *out_width = w.hand_normalize(dpi_x, 0.0, 12.0).round() as i32;
+                    *out_height = h.hand_normalize(dpi_y, 0.0, 12.0).round() as i32;
+                }
+                true.to_glib()
+            }
+            (_, _, _) => false.to_glib(),
+        },
+    )
 }
 
 #[no_mangle]
