@@ -107,7 +107,7 @@ impl<'a> Iterator for Pixels<'a> {
 impl<'a> Iterator for PixelRectangle<'a> {
     type Item = (i32, i32, Pixel);
 
-    #[inline]
+    #[inline(always)]
     fn next(&mut self) -> Option<Self::Item> {
         // This means we hit the end on the last iteration.
         if self.x == self.rectangle.x1 || self.y == self.rectangle.y1 {
@@ -171,6 +171,7 @@ impl<'a> Iterator for PixelRectangle<'a> {
 mod tests {
     use super::*;
     use cairo::{self, ImageSurface};
+    use surface_utils::shared_surface::SurfaceType;
 
     #[test]
     fn pixels_count() {
@@ -179,6 +180,7 @@ mod tests {
 
         let surface = SharedImageSurface::new(
             ImageSurface::create(cairo::Format::ARgb32, WIDTH, HEIGHT).unwrap(),
+            SurfaceType::SRgb,
         ).unwrap();
 
         // Full image.
@@ -255,6 +257,7 @@ mod tests {
 
         let surface = SharedImageSurface::new(
             ImageSurface::create(cairo::Format::ARgb32, WIDTH, HEIGHT).unwrap(),
+            SurfaceType::SRgb,
         ).unwrap();
 
         let bounds = IRect {
